# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

from getpaid.abstract_mixin import AbstractMixin
from model_utils.choices import Choices

from . import PaymentProcessor


EPAY_STATUSES = Choices(
    (0, "new", _("waiting for payment")),
    (1, "inblock", _("money are blocked")),
    (2, "captured", _("paid")),
    (3, "cancelled", _("cancelled")),
    (4, "refunded", _("refunded")),
    (5, "failed", _("failed")),
)


def int_or_none(val):
    try:
        return int(val)
    except ValueError:
        return None


class EpayPaymentFactory(models.Model, AbstractMixin):
    """

    "@name": "GERMAN ILIN",
    "@mail": "germanilyin@gmail.com",
    "@phone": "",

    "@merchant_id": "92061101",
    "@card": "548318-XX-XXXX-0293",
    "@amount": "1000.0000",
    "@reference": "160216002802",
    "@approval_code": "002802",
    "@response_code": "00",
    "@Secure": "Yes",
    "@card_bin": "KAZ",
    "@c_hash": "D643983890D0003EA973E88A346CDDBE"

    """

    statuses = EPAY_STATUSES

    merchant_id = models.CharField(max_length=15, blank=True)
    card = models.CharField(max_length=len("548318-XX-XXXX-0293"), blank=True)
    reference = models.CharField(max_length=15, blank=True)
    approval_code = models.CharField(max_length=15, blank=True)
    response_code = models.CharField(max_length=5, blank=True)
    is_secure = models.BooleanField(default=True, blank=True)
    card_bin = models.CharField(max_length=10, blank=True)
    c_hash = models.CharField(max_length=len("D643983890D0003EA973E88A346CDDBE") + 4, blank=True)
    customer_name = models.CharField(max_length=50, blank=True)
    customer_mail = models.CharField(max_length=75, blank=True)
    customer_phone = models.CharField(max_length=20, blank=True)
    is_payment = models.BooleanField(default=True)
    status = models.IntegerField(blank=True, null=True)
    result = models.IntegerField(blank=True, null=True)

    class Meta:
        abstract = True

    @classmethod
    def contribute(cls, payment):
        return {'payment': models.OneToOneField(payment)}

    def update_from_kkb(self, response=None):
        if not response:
            response = PaymentProcessor.get_status(self.payment_id)
        self.is_payment = response['payment'].lower() == "true"
        self.status = int_or_none(response['status'])
        self.result = int_or_none(response['result'])
        self.approval_code = response['approval_code']
        self.ping_status()
        self.save()

    def ping_status(self):
        new_status = self.get_status()
        if new_status == self.statuses.cancelled:
            self.payment.change_status('cancelled')
        if new_status == self.statuses.failed:
            self.payment.change_status('failed')
        if new_status == self.statuses.refunded:
            self.payment.change_status('cancelled')
        if new_status == self.statuses.new:
            self.payment.change_status('in_progress')
        if new_status == self.statuses.captured:
            self.payment.change_status('paid')

    @staticmethod
    def import_or_update(response, payment_id):
        try:
            epay_payment = EpayPayment.objects.select_related('payment').get(payment_id=payment_id)
            epay_payment.update_from_kkb(response)
        except EpayPayment.DoesNotExist:
            raise NotImplementedError("Need to implement import part")

    def get_status(self):
        return EpayPaymentFactory.calc_status(self.is_payment, self.status, self.result)

    @staticmethod
    def calc_status(payment, status, result):
        if not payment:
            if status == 2:
                return EPAY_STATUSES.cancelled
            return EPAY_STATUSES.failed
        if payment:
            if status == 0:
                return EPAY_STATUSES.new
            if status == 2:
                return EPAY_STATUSES.captured


EpayPayment = None


def build_models(payment_class):
    global EpayPayment

    class EpayPayment(EpayPaymentFactory.construct(payment_class)):
        pass
    return [EpayPayment]



from django.apps import apps
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from epay.kkb.exceptions import EpayError
from . import PaymentProcessor


def epay_payment_status(obj):
    return '<a href="{}">{}</a>'.format(
        reverse("epay:status", kwargs={'pk': obj.id}),
        _("Check status")
    )

epay_payment_status.allow_tags = True
epay_payment_status.short_description = _("EPAY status")


def complete_epay_payment(modeladmin, request, queryset):
    payment = queryset.select_related('epaypayment').first()

    try:
        payment.epaypayment
    except ObjectDoesNotExist:
        messages.error(request, _("This payment has no epay payment associated with it"))
        return None

    try:
        PaymentProcessor.completed(payment.id)
        messages.info(request, _("Payment successfully completed"))
    except EpayError as e:
        messages.error(request, _("Request failed: %s") % unicode(e))
complete_epay_payment.short_description = _("Complete epay payment")


def refund_epay_payment(modeladmin, request, queryset):
    payment = queryset.select_related('epaypayment').first()

    try:
        epay_payment = payment.epaypayment
    except ObjectDoesNotExist:
        messages.error(request, _("This payment has no epay payment associated with it"))
        return None

    try:
        PaymentProcessor.refunded(payment.id)
        messages.info(request, _("Payment successfully refunded"))
    except EpayError as e:
        messages.error(request, _("Request failed: %s") % unicode(e))

refund_epay_payment.short_description = _("Refund epay payment")


def reverse_epay_payment(modeladmin, request, queryset):
    payment = queryset.select_related('epaypayment').first()

    try:
        epay_payment = payment.epaypayment
    except ObjectDoesNotExist:
        messages.error(request, _("This payment has no epay payment associated with it"))
        return None

    try:
        PaymentProcessor.reversed(payment.id)
        messages.info(request, _("Payment successfully reversed"))
    except EpayError as e:
        messages.error(request, _("Request failed: %s") % unicode(e))

reverse_epay_payment.short_description = _("Reverse epay payment")


def check_epay_payment(modeladmin, request, queryset):
    payment = queryset.select_related('epaypayment').first()
    EpayPayment = apps.get_model('epay', 'epaypayment')

    try:
        payment.epaypayment
    except ObjectDoesNotExist:
        EpayPayment.objects.create(payment=payment)

    try:
        PaymentProcessor.update_status(payment.id)
        messages.info(request, _("Payment successfully updated"))
    except EpayError as e:
        messages.error(request, _("Request failed: %s") % unicode(e))

check_epay_payment.short_description = _("Check epay payment")



import requests

from . import utils
from . import signing
from . import exceptions


def id_modify(self, id):
    return id


class Epay(object):

    BACKEND_TEST_GATEWAY_BASE_URL = u'https://testpay.kkb.kz'
    BACKEND_GATEWAY_BASE_URL = u'https://epay.kkb.kz'
    BACKEND_POST_URI = '/jsp/process/logon.jsp'
    BACKEND_CONFIRM_URI = '/jsp/remote/control.jsp'
    BACKEND_CHECK_URI = '/jsp/remote/checkOrdern.jsp'
    BACKEND_ACCEPTED_CURRENCY_DICT = {
        'KZT': 398,
        'USD': 840,
        'EUR': 978,
    }
    BACKEND_ACCEPTED_CURRENCY = (u'KZT', u'USD', u'EUR')

    modify_order_id = id_modify
    unmodify_order_id = id_modify
    key_passphrase = None
    testing = False
    private_key = None
    kkb_key = None
    merchant_name = "Demo Shop"
    merchant_id = "92061101"
    merchant_cert_id = "00c182b189"
    auto_capture = False

    def __init__(self, **kw):
        for key, value in kw.items():
            if key.startswith('get_'):
                continue
            if key in ('modify_order_id', 'unmodify_order_id'):
                setattr(self, key, lambda self, id: value(id))
            setattr(self, key, value)

    def get_number_for_currency(self, currency):
        return self.BACKEND_ACCEPTED_CURRENCY_DICT.get(currency, None)

    def sign_order(self, payment_id, amount, currency="KZT"):
        return utils.b64(utils.payment_auth(
            order_id=self.modify_order_id(payment_id),
            amount=amount,
            currency_code=self.get_number_for_currency(currency),
            merchant_id=self.merchant_id,
            merchant_name=self.merchant_name,
            cert_id=self.merchant_cert_id,
            private_key=self.private_key,
            passphrase=self.key_passphrase,
        ))

    def get_command_url(self, command, payment_id, amount, approval_code, reference, currency="KZT", reason=None):
        """

        :param command: one of complete, reverse, refund
        :param payment_id:
        :param amount:
        :param approval_code:
        :param reference:
        :param currency:
        :param reason:
        :return:
        """
        quoted_xml = utils.urlquote(utils.payment_control(
            command,
            reference=reference,
            approval_code=approval_code,
            order_id=self.modify_order_id(payment_id),
            amount=amount,
            reason=reason,
            currency_code=self.get_number_for_currency(currency),
            merchant_id=self.merchant_id,
            private_key=self.private_key,
            passphrase=self.key_passphrase,
        ))
        url = self.get_gateway_base_url() + self.BACKEND_CONFIRM_URI + "?" + quoted_xml
        if self.testing:
            url = url.replace('https', 'http')
        return url

    def get_status_url(self, payment_id):
        quoted_xml = utils.urlquote(utils.payment_status(
            order_id=self.modify_order_id(payment_id),
            merchant_id=self.merchant_id,
            private_key=self.private_key,
            passphrase=self.key_passphrase,
        ))
        url = self.get_gateway_base_url() + self.BACKEND_CHECK_URI + "?" + quoted_xml
        if self.testing:
            url = url.replace('https', 'http')
        return url

    def get_gateway_base_url(self):
        if self.testing:
            return self.BACKEND_TEST_GATEWAY_BASE_URL
        return self.BACKEND_GATEWAY_BASE_URL

    def handle_response(self, response):
        result, signature, letter = utils.parse_response(response)
        if 'error' in result:
            raise exceptions.RequestError(result['error'])
        if not signing.verify_sign(signature, letter, self.kkb_key):
            raise exceptions.VerificationError("Failed to verify signature")
        return utils.get_bank(result)

    def get_gateway_url(self):
        return self.get_gateway_base_url() + self.BACKEND_POST_URI

    def capture(self, payment_id, amount, approval_code, reference, currency="KZT"):
        url = self.get_command_url("complete", payment_id, amount, approval_code, reference, currency)
        response = requests.get(url)
        response = self.handle_response(response.content)
        if response['response']['code'] != '00':
            raise exceptions.EpayError(response['response']['message'])
        return response

    def get_status(self, payment_id):
        url = self.get_status_url(payment_id)
        response = requests.get(url)
        response = self.handle_response(response.content)
        if 'response' not in response:
            raise exceptions.EpayError("Something is wrong")
        return response

    def refund(self, payment_id, amount, approval_code, reference, currency="KZT"):
        url = self.get_command_url(
            "refund", payment_id, amount, approval_code, reference, currency, reason="Client initiative")
        response = requests.get(url)
        response = self.handle_response(response.content)
        if response['response']['code'] != '00':
            raise exceptions.EpayError(response['response']['message'])
        return response

    def cancel(self, payment_id, amount, approval_code, reference, currency="KZT"):
        url = self.get_command_url(
            "reverse", payment_id, amount, approval_code, reference, currency, reason="Client initiative")
        response = requests.get(url)
        print(response.status_code, response.content, url)
        response = self.handle_response(response.content)
        if response['response']['code'] != '00':
                raise exceptions.EpayError(response['response']['message'])

        return response



# -*- coding: utf-8 -*-

import logging

from django.apps import apps
from django.http import HttpResponse, HttpResponseBadRequest
from django.views.generic import View
from django.views.generic.detail import DetailView

from epay.kkb import exceptions
from . import PaymentProcessor

logger = logging.getLogger(__name__)


class EpayCallback(View):
    http_method_names = ['post']

    def post(self, request, *args, **kwargs):
        if 'response' not in request.POST:
            return HttpResponseBadRequest("400 Bad Request")
        try:
            PaymentProcessor.callback(request.POST['response'])
            return HttpResponse('OK')
        except exceptions.VerificationError:
            return HttpResponseBadRequest("400 Bad Request")
        except exceptions.EpayError:
            return HttpResponseBadRequest("400 Bad Request")


class PaymentDetail(DetailView):
    model = apps.get_model("getpaid", "payment")
    template_name = 'epay/payment_detail.html'

    def get_queryset(self):
        qs = super(PaymentDetail, self).get_queryset()
        return qs.select_related('epaypayment')

    def get_context_data(self, object, **kwargs):
        ctx = super(PaymentDetail, self).get_context_data(object=object, **kwargs)
        ctx.update({
            "response": PaymentProcessor.get_status(object.pk)
        })
        return ctx



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-18 07:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import getpaid.abstract_mixin


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('getpaid', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EpayPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('merchant_id', models.CharField(blank=True, max_length=15)),
                ('card', models.CharField(blank=True, max_length=19)),
                ('reference', models.CharField(blank=True, max_length=15)),
                ('approval_code', models.CharField(blank=True, max_length=15)),
                ('response_code', models.CharField(blank=True, max_length=5)),
                ('is_secure', models.BooleanField(default=True)),
                ('card_bin', models.CharField(blank=True, max_length=10)),
                ('c_hash', models.CharField(blank=True, max_length=36)),
                ('customer_name', models.CharField(blank=True, max_length=50)),
                ('customer_mail', models.CharField(blank=True, max_length=75)),
                ('customer_phone', models.CharField(blank=True, max_length=20)),
                ('is_payment', models.BooleanField(default=True)),
                ('status', models.IntegerField(blank=True, null=True)),
                ('result', models.IntegerField(blank=True, null=True)),
                ('payment', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='getpaid.Payment')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, getpaid.abstract_mixin.AbstractMixin),
        ),
    ]



import M2Crypto
import base64


def sign_string(string, private_key, passphrase=None):
    signEVP = M2Crypto.EVP.load_key_string(private_key, callback=lambda s: passphrase)
    signEVP.sign_init()
    signEVP.sign_update(string)
    signature = signEVP.sign_final()
    signature = signature[::-1]
    return base64.encodestring(signature)


def verify_sign(sign, letter, public_key):
    rawsign = base64.decodestring(sign)[::-1]
    pubkey = M2Crypto.X509.load_cert_string(public_key).get_pubkey()
    pubkey.verify_init()
    pubkey.verify_update(letter)
    return pubkey.verify_final(rawsign)



# -*- coding: utf-8 -*-
from django.conf.urls import include, patterns, url

from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import user_passes_test
from django.db import transaction

from .views import EpayCallback, PaymentDetail
from . import PaymentProcessor

has_perm = PaymentProcessor.get_backend_setting("has_perm", lambda u: u.is_superuser)


my_urls = patterns(
    '',
    url(r'^callback/$', csrf_exempt(require_POST(transaction.atomic(EpayCallback.as_view()))), name='postlink'),
    url(r'^status/(?P<pk>\d+)/$', user_passes_test(has_perm)(PaymentDetail.as_view()), name='status'),
)


urlpatterns = patterns(
    '',
    url(r'', include(my_urls, namespace='epay'))
)



import os
from decimal import Decimal

from django.apps import apps
from django.conf import settings
from django.db import transaction
from django.utils.translation import ugettext_lazy as _
from getpaid.backends import PaymentProcessorBase
from getpaid.utils import build_absolute_uri, get_domain, get_backend_settings

from .kkb.processing import Epay

__version__ = '0.1.4'


DEFAULT_KKB_PUB_KEY = os.path.join(os.path.dirname(__file__), "keys", "kkbca.pem")
DEFAULT_MERCHANT_PRIVATE_KEY = os.path.join(os.path.dirname(__file__), "keys", "cert.pem")


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'epay'
    BACKEND_NAME = _('Kazkom Epay')

    BACKEND_ACCEPTED_CURRENCY_DICT = {
        'KZT': 398,
        'USD': 840,
        'EUR': 978,
    }
    BACKEND_ACCEPTED_CURRENCY = (u'KZT', u'USD', u'EUR')

    BACKEND_LOGO_URL = os.path.join(settings.STATIC_URL, 'epay/img/logo.png')

    backend_settings = get_backend_settings(BACKEND)
    epay = Epay(
        private_key=open(backend_settings.get(
            'merchant_private_key', DEFAULT_MERCHANT_PRIVATE_KEY), 'rb').read(),
        kkb_key=open(backend_settings.get(
            'kkb_pub_key', DEFAULT_KKB_PUB_KEY), 'rb').read(),
        **backend_settings
    )

    def get_scheme(self, request):
        scheme = self.get_backend_setting('scheme', 'https')
        if not scheme:
            if request.is_secure():
                scheme = 'https'
            else:
                scheme = 'http'
        return scheme

    def get_language(self, request):
        """
        :return: rus|eng depending on user request
        """
        if 'get_language' in self.backend_settings:
            return self.backend_settings['get_language'](request, self.payment)
        return "rus"

    def get_email(self, request):
        if 'get_email' in self.backend_settings:
            return self.backend_settings['get_email'](request, self.payment)
        if hasattr(self.payment.order, 'email'):
            return self.payment.order.email

    def get_template(self, request):
        if 'get_template' in self.backend_settings:
            return self.backend_settings['get_template'](request, self.payment)
        if 'template' in self.backend_settings:
            return self.backend_settings['template']

    def get_fail_backlink(self, request):
        if 'get_fail_backlink' in self.backend_settings:
            return self.backend_settings['get_fail_backlink'](request, self.payment)

    def build_attrs(self, request, scheme='https', language="rus"):
        attrs = {
            'Signed_Order_B64': self.epay.sign_order(self.payment.id, self.payment.amount, self.payment.currency),
            'Language': language,
            'BackLink': self.get_backlink_url(request, scheme=scheme),
            'PostLink': self.get_postlink_url(request, scheme=scheme)
        }
        email = self.get_email(request)
        template = self.get_template(request)
        fail_back_link = self.get_fail_backlink(request)
        if email:
            attrs['email'] = email

        if template:
            attrs['template'] = template

        if fail_back_link:
            attrs['FailureBackLink'] = fail_back_link

        # TODO: support for FailurePostLink

        return attrs


    def get_gateway_url(self, request):
        return (
            self.epay.get_gateway_url(),
            "POST",
            self.build_attrs(request, self.get_scheme(request), language=self.get_language(request))
        )

    def get_backlink_url(self, request, scheme):
        if 'get_backlink_url' in self.backend_settings:
            return self.backend_settings['get_backlink_url'](request, self.payment)
        return "{}://{}{}".format(
            scheme,
            get_domain(),
            self.payment.order.get_absolute_url())

    def get_postlink_url(self, request, **kw):
        return build_absolute_uri('epay:postlink', **kw)

    @staticmethod
    def amount_to_python(amount_str):
        return Decimal(amount_str)

    @staticmethod
    def callback(response):
        """
        Payment was confirmed.
        """
        epay = PaymentProcessor.epay
        Payment = apps.get_model('getpaid', 'Payment')
        EpayPayment = apps.get_model('epay', 'EpayPayment')
        with transaction.atomic():
            params = epay.handle_response(response)
            customer_params = params['customer']
            payment_params = params['results']['payment']
            order_params = customer_params['merchant']['order']
            payment_id = epay.unmodify_order_id(int(order_params['order_id']))

            payment = Payment.objects.get(id=payment_id)
            assert payment.status == 'in_progress',\
                "Can not confirm payment that was not accepted for processing"
            payment.external_id = payment_params['reference']
            amount = PaymentProcessor.amount_to_python(payment_params['amount'])
            payment.amount_paid = amount

            epay_payment = EpayPayment(
                payment=payment,
                merchant_id=payment_params.get('merchant_id', ''),
                card=payment_params.get('card', ''),
                reference=payment_params.get('reference', ''),
                approval_code=payment_params.get('approval_code', ''),
                response_code=payment_params.get('response_code', ''),
                is_secure=(payment_params.get('secure', 'No') == 'Yes'),
                card_bin=payment_params.get('card_bin', ''),
                c_hash=payment_params.get('c_hash', ''),
                customer_name=customer_params.get('name', ''),
                customer_mail=customer_params.get('mail', ''),
                customer_phone=customer_params.get('phone', ''),
            )
            epay_payment.save()
            res = payment.on_success(amount)
            if epay.auto_capture:
                PaymentProcessor.completed(epay_payment.pk)
            return res



    @staticmethod
    def completed(payment_id):
        """
        Confirming payment. Money is transfered from card to our account
        """
        epay = PaymentProcessor.epay
        EpayPayment = apps.get_model('epay', 'EpayPayment')
        with transaction.atomic():
            epay_payment = EpayPayment.objects.select_related('payment').get(payment_id=payment_id)
            payment = epay_payment.payment
            epay.capture(
                payment_id, payment.amount, epay_payment.approval_code,
                epay_payment.reference, currency=payment.currency)

            # epay_payment.change_status("caputred")
            epay_payment.update_from_kkb()
            return epay_payment

    @staticmethod
    def reversed(payment_id):
        """
        Reversing money if they are still in block
        """
        EpayPayment = apps.get_model('epay', 'EpayPayment')
        epay = PaymentProcessor.epay
        with transaction.atomic():
            epay_payment = EpayPayment.objects.select_related('payment').get(payment_id=payment_id)
            payment = epay_payment.payment
            epay.cancel(
                payment_id, payment.amount, epay_payment.approval_code,
                epay_payment.reference, currency=payment.currency)

            payment.change_status("cancelled")

            return epay_payment

    @staticmethod
    def refunded(payment_id):
        """
        Reversing money if they are still in block
        """
        EpayPayment = apps.get_model('epay', 'EpayPayment')
        epay = PaymentProcessor.epay
        with transaction.atomic():
            epay_payment = EpayPayment.objects.select_related('payment').get(payment_id=payment_id)
            payment = epay_payment.payment
            epay.refund(
                payment_id, payment.amount, epay_payment.approval_code,
                epay_payment.reference, currency=payment.currency)

            payment.change_status("cancelled")

            return epay_payment

    @staticmethod
    def get_status(payment_id):
        epay = PaymentProcessor.epay
        response = epay.get_status(payment_id)
        return response['response']

    @staticmethod
    def update_status(payment_id):
        """
        Reversing money if they are still in block
        """
        EpayPayment = apps.get_model('epay', 'EpayPayment')
        with transaction.atomic():
            response = PaymentProcessor.get_status(payment_id)
            epay_payment = EpayPayment.import_or_update(response, payment_id=payment_id)
            return epay_payment



import base64
try:
    from urllib.parse import quote
except:
    from urllib import quote

import xmltodict

from .signing import sign_string


ORDER_TPL = (
    """<merchant cert_id="{cert_id}" name="{merchant_name}">"""
    """<order order_id="{order_id:06d}" amount="{amount}" currency="{currency_code}">"""
    """<department merchant_id="{merchant_id}" amount="{amount}"/></order></merchant>""")


COMMAND_TPL = (
    """<merchant id="{merchant_id}"><command type="{command}"/>"""
    """<payment reference="{reference_id}" approval_code="{approval_code}" orderid="{order_id}" """
    """amount="{amount}" currency_code="{currency_code}"/><reason>{reason}</reason></merchant>""")


STATUS_TPL = """<merchant id="{merchant_id}"><order id="{order_id}"/></merchant>"""


DOC_TPL = ("""<document>{document}<merchant_sign type="RSA">{signature}</merchant_sign></document>""")


def payment_auth(order_id, amount, currency_code, cert_id, merchant_id, merchant_name, private_key, passphrase):
    """
        <document>
            <merchant cert_id="1231232" name="Test Shop">
                <order order_id="000001" amount="1000.00" currency="398">
                    <department merchant_id="23124" amount="1000.00"></department>
                </order>
            </merchant>
            <merchant_sign type="RSA">SIGNATURE</merchant_sign>
        </document>

    :param order_id:
    :param amount:
    :param currency_code:
    :param cert_id:
    :param merchant_id:
    :param merchant_name:
    :return: base64 encoded signed xml document
    """
    order_xml = ORDER_TPL.format(
        order_id=order_id,
        amount=amount,
        cert_id=cert_id,
        currency_code=currency_code,
        merchant_name=merchant_name,
        merchant_id=merchant_id
    )
    signature = sign_string(order_xml, private_key=private_key, passphrase=passphrase)
    document_xml = DOC_TPL.format(document=order_xml, signature=signature)
    return document_xml


def payment_control(
        command, reference, approval_code, order_id, amount, currency_code,
        merchant_id, private_key, passphrase=None, reason=None):
    if not reason:
        reason = ''
    order_xml = COMMAND_TPL.format(
        command=command,
        order_id=order_id,
        reference_id=reference,
        approval_code=approval_code,
        amount=amount,
        reason=reason,
        currency_code=currency_code,
        merchant_id=merchant_id)
    signature = sign_string(order_xml, private_key=private_key, passphrase=passphrase)
    document_xml = DOC_TPL.format(document=order_xml, signature=signature)
    print(document_xml)
    return document_xml


def payment_status(order_id, merchant_id, private_key, passphrase=None):
    order_xml = STATUS_TPL.format(order_id=order_id, merchant_id=merchant_id)
    signature = sign_string(order_xml, private_key=private_key, passphrase=passphrase)
    document_xml = DOC_TPL.format(document=order_xml, signature=signature)
    return document_xml


def decode(value):
    return value.decode('string_escape')


def parse(xml):
    """

    :param value:
    :return: an ordered dict
    """
    # dirty hack around EPAY bug
    # KKB, what a hell? <p><Error></Error>
    if '<p>' in xml:
        xml = '<hack>{}</hack>'.format(xml.replace('<p>', ''))
    res = xmltodict.parse(
        xml, attr_prefix='', cdata_key='body',
        postprocessor=lambda path, key, value: (key.lower(), value))
    if 'hack' in res:
        return res['hack']
    return res


_BANK_OFFSET = len('</bank>')


def get_letter(xml):
    """

    :param xml:
    :return: everything between <bank> tag
    """
    try:
        left, right = xml.index('<bank '), xml.index('</bank>') + _BANK_OFFSET
        return xml[left:right]
    except ValueError:
        return None


def get_bank_signature(doc):
    if 'bank_sign' in doc['document']:
        return doc['document']['bank_sign']['body']
    # dirty hack around strange KKB response format
    if 'merchant' in doc['document'] and 'bank_sign' in doc['document']['merchant']:
        return doc['document']['merchant']['bank_sign']['body']


def get_bank(doc):
    if 'bank' in doc:
        return doc['bank']
    # dirty hack around strange KKB response format
    if 'merchant' in doc and 'bank' in doc['merchant']:
        return doc['merchant']['bank']


def parse_response(value):
    response = decode(value)
    result = parse(response)
    if 'error' in result:
        return result, None, None
    return result['document'], get_bank_signature(result), get_letter(response)


def b64(value):
    return base64.encodestring(value)


def urlquote(value):
    return quote(value)


# def request_url(link):
#     req = urllib.Request(link)
#     sock = urllib.urlopen(req)
#     response = sock.read()
#     sock.close()
#     return response.strip()


def _print(obj):
    import json
    print(json.dumps(obj, indent=2))
